/***********************************************************

	pic24timer.c

	Version 0.91: 2009/02/12

	Copyright (C) 2008-2009, MicroFan Div., PeopleMedia, Inc.
	All rights reserved.

***********************************************************/

#include "picsys24.h"


/***********************************************************
	v
***********************************************************/

static volatile unsigned long millisec = 0 ;


void
delayMicroseconds(unsigned int t)
{
	while (t-- != 0) {
#if F_CPU == 32 MHz
		Nop() ; Nop() ; Nop() ; Nop() ;
		Nop() ; Nop() ; Nop() ; Nop() ;
		Nop() ; Nop() ; Nop() ; Nop() ;
#elif F_CPU == 16 MHz
		Nop() ; Nop() ; Nop() ; Nop() ;
#elif F_CPU <= 8 MHz
		/* nothing */
#endif
	}
}


void
delayMilliseconds(unsigned int t)
{
	unsigned long s ;

	s = millisec + t ;

	while (millisec <= s) {
		Idle() ;
	}	

	return ;
}


unsigned long
millis()
{
	return millisec ;
}


/***********************************************************
	^C}[荞݊֌W
***********************************************************/

/* T[{p */
void
TIMER5_setup(int v)
{
	unsigned int cnt ;

	cnt = SERVO_NEUTRAL + v * (SERVO_MINMAX/100) ;
	PR5 = (cnt * (unsigned int)(F_CPU/(2 MHz))) ;

	TMR5 = 0 ;

	IPC7bits.T5IP = 5 ;	// x
	IFS1bits.T5IF = 0 ;
	IEC1bits.T5IE = 1 ;
}


/* VXe̒jɂȂԊu荞 */
void
TIMER4_init()
{
	PR4 = F_CPU / 2 / HEART_BEAT - 1 ;

	IPC6bits.T4IP = 3 ;	// ჌x
	IFS1bits.T4IF = 0 ;
	IEC1bits.T4IE = 1 ;

	T4CONbits.TON = 1 ;
}


/***********************************************************
	荞݃nh
***********************************************************/

extern byte _beepPin ;
extern byte _beepValue ;

extern byte _pwmPin[]  ;
extern byte _pwmValue[] ;

extern byte _servoPin[]  ;
extern byte _servoDisable ;
extern int _servoValue[] ;
extern int _servoMin[] ;
extern int _servoMax[] ;

static byte __servoNum ;


/* T[{pXIp荞݃nh */
// void _ISRFAST
void __attribute__((__interrupt__, __shadow__, __auto_psv__))
_T5Interrupt()
{	
	/* ServoPWMpXI */
	isr_bitOut(_servoPin[__servoNum], LOW) ;
	/* 蓖ĂĂ΂̏ */
	if (_servoDisable & BIT(__servoNum)) {
		_servoPin[__servoNum] = NO_ASSIGN ;
		_servoDisable &= ~BIT(__servoNum) ;
	}

	/* TIMER3͂񂨋x */
	T5CONbits.TON = 0 ;
	IEC1bits.T5IE = 0 ;
	IFS1bits.T5IF = 0 ;

#if defined(DOZE_MODE2) || defined(DOZE_MODE4) || defined(DOZE_MODE8) || defined(DOZE_MODE16) || defined(DOZE_MODE32)
	CLKDIVbits.DOZEN = 1 ; 
#endif
}


byte _refreshMask ;

extern void _uxRxPolling() ;

/* 荞݃nh */
// void _ISR
void __attribute__((__interrupt__, __auto_psv__))
_T4Interrupt()
{
	static byte n ;
	static byte sv = 0 ; // T[{pX̃X^[g^C~OǗ
	byte i, id ;

	IFS1bits.T4IF = 0 ;

	/* ɂȂ悤ɂł邾 */
	if (_beepPin != NO_ASSIGN)
		isr_bitOut(_beepPin, n & BIT(8 - _beepValue)) ;

	_uxRxPolling() ;

	n++ ;

#if HEART_BEAT == 4 KHz
	if ((n & 0b11) == 0) {
		millisec++ ;
	}
#elif HEART_BEAT == 2 KHz
	if ((n & 1) == 0) {
		millisec++ ;
	}
#else
	millisec++ ;
#endif
	sv++ ;

	if (sv >= 20*(HEART_BEAT/(1 KHz)))	// 20msłP
		sv = 0 ;

	i = n & (BIT(4) * (HEART_BEAT/(1 KHz)) - 1) ;

	if (_pwmPin[0] != NO_ASSIGN)
		isr_bitOut(_pwmPin[0], _pwmValue[0] > i) ;

	if (_pwmPin[1] != NO_ASSIGN)
		isr_bitOut(_pwmPin[1], _pwmValue[1] > i) ;

	if (_pwmPin[2] != NO_ASSIGN)
		isr_bitOut(_pwmPin[2], _pwmValue[2] > i) ;

	if (_pwmPin[3] != NO_ASSIGN)
		isr_bitOut(_pwmPin[3], _pwmValue[3] > i) ;

	id = NO_ASSIGN ;
	if ((sv == 0*(HEART_BEAT/(1 KHz))) && (_servoPin[0] != NO_ASSIGN)) {
		id = 0 ;
	} else if ((sv == 5*(HEART_BEAT/(1 KHz))) && (_servoPin[1] != NO_ASSIGN)) {
		id = 1 ;
	} else if ((sv == 10*(HEART_BEAT/(1 KHz))) && (_servoPin[2] != NO_ASSIGN)) {
		id = 2 ;
	} else if ((sv == 15*(HEART_BEAT/(1 KHz))) && (_servoPin[3] != NO_ASSIGN)) {
		id = 3 ;
	}

	if (id != NO_ASSIGN) {
		__servoNum = id ;
		TIMER5_setup(_servoValue[id]) ;

		/* ^C~OǗF̂Q̏͊荞݃nh̏Ƃ킹 */
		T5CONbits.TON = 1 ;	// 
#if F_CPU <= 8 MHz
//		delayMicroseconds(10) ;
#endif
		isr_bitOut(_servoPin[id], HIGH) ; // ͌
	}

	/* _Ci~bN\XV */
#if defined(USE_SEG7)
	if ((n & (BIT(HEART_BEAT/(1 KHz)+1) - 1)) == 0)
		isr_SEG7_update() ;	// 250Hz:1Cycle-62.5Hz
#elif defined(USE_DML)
	if ((n & (BIT(HEART_BEAT/(1 KHz)) - 1)) == 0)
		isr_DML_update() ;	// 500Hz:1Cycle-62.5Hz
#elif defined(DUO_GL128_PIC24F)
	if ((n & (BIT(HEART_BEAT/(1 KHz)+1) - 1)) == 0)
		isr_SW_update() ;	// 250Hz
#endif

#if defined(DOZE_MODE2) || defined(DOZE_MODE4) || defined(DOZE_MODE8) || defined(DOZE_MODE16) || defined(DOZE_MODE32)
	CLKDIVbits.DOZEN = 1 ; 
#endif
}

